import os
import grok
from grok import index
from hurry.query.query import Query, Text
from hurry.query import Eq
from waeup.app import University
from waeup.interfaces import ICourse
from waeup.student.interfaces import IStudent
from zope.app.catalog.interfaces import ICatalog
from zope.app.intid.interfaces import IIntIds
from zope.component import getUtility
from zope.component.interfaces import ComponentLookupError


class StudentIndexes(grok.Indexes):
     grok.site(University)
     grok.name('students_catalog')
     grok.context(IStudent)

     name = index.Field(attribute='name')


class StudentSearch(grok.View):
    grok.context(University)

    def update(self, query=None):
        self.search_result = []
        if query is not None:
            self.search_result = self.search_context(query)
            print "RESULT: ", list(self.search_result)

    def search_context(self, query):
        result = Query().searchResults(
            Eq(('students_catalog', 'name'), query)
            )
        return result
    
class CourseIndexes(grok.Indexes):
     grok.site(University)
     grok.name('courses_catalog')
     grok.context(ICourse)

     code = index.Field(attribute='code')
     title = index.Text(attribute='title')


@grok.subscribe(ICourse, grok.IObjectAddedEvent)
def handleCourseAdd(obj, event):
     """Index an added course with the local catalog.

     Courses are not indexed automatically, as they are not a
     dictionary subitem of the accompanied site object
     (`IUniversity`). I.e. one cannot get them by asking for
     ``app['FACCODE']['DEPTCODE']['COURSECODE']`` but one has to ask for
     ``app.faculties['FACCODE']['DEPTCODE'].courses['COURSECODE']``.

     Once, a course is indexed we can leave the further handling to
     the default component architechture. At least removals will
     be handled correctly then (and the course unindexed).
     """
     try:
          cat = getUtility(ICatalog, name='courses_catalog')
     except ComponentLookupError:
          # catalog not available. This might happen during tests.
          return
     intids = getUtility(IIntIds)
     index = cat['code']
     index.index_doc(intids.getId(obj), obj)


from zope.interface import Interface
from zope import schema

class IQueryResultItem(Interface):
     url = schema.TextLine(
          title = u'URL that links to the found item')
     title = schema.TextLine(
          title = u'Title displayed in search results.')
     description = schema.Text(
          title = u'Longer description of the item found.')
     
     
class QueryResultItem(object):
     grok.implements(IQueryResultItem)
     url = None
     title = None
     description = None
     
     def __init__(self, context, view):
          self.context = context
          self.url = view.url(context)
          self.title = context.title
          self.description = ''
     
class CourseQueryResultItem(QueryResultItem):
     def __init__(self, context, view):
          self.context = context
          self.url = view.url(context)
          self.title = "COURSE: " + context.title
          self.description = 'code: %s' % context.code


from waeup.viewlets import Search, MainArea
from hurry.query.query import Query, Text, Eq

class SearchViewlet(grok.Viewlet):
     grok.viewletmanager(MainArea)
     grok.view(Search)
     grok.context(University)

     def update(self, *args, **kw):
          #print "UPDATE: ", args, kw
          #print self.request.form
          form = self.request.form
          self.hitlist = []
          self.query = ''
          if not 'query' in form:
               return
          query = form['query']
          self.query = query
          cat = getUtility(ICatalog, name='courses_catalog')
          #print "CODES: ",  cat['code'].documentCount()
          #print "TITLES: ", cat['title'].documentCount()
          results = list(cat.searchResults(code=(query, query)))#
          #title=(query, query)))
          hitlist = []
          results = Query().searchResults(
               Eq(('courses_catalog', 'code'), query))
          for result in results:
               hitlist.append(CourseQueryResultItem(result, view=self.view))

          results = Query().searchResults(
               Text(('courses_catalog', 'title'), query))
          
          for result in results:
               hitlist.append(CourseQueryResultItem(result, view=self.view))

          self.hitlist = hitlist
